import argparse
import re
from Bio import Entrez, SeqIO

parser = argparse.ArgumentParser(description="Download sequences from Genbank programmatically")
parser.add_argument('-r', '--rna', default='rna.fna', help='input RNA file for 3 \' UTR processing')
parser.add_argument('-c', '--cds', default='cds.fna', help='CDS file to find gene length from mRNA FASTA (--input)')
parser.add_argument('-f', '--format', default='fasta', help='sequence input format (fasta or genbank)')
parser.add_argument('-o', '--outfile', default='fetched-utr.fasta', help='file target for processed sequences')

args = parser.parse_args()

output_handle = open(args.outfile, 'w')
no_utr_handle = open('.'.join([args.rna, 'no-utr.txt']), 'w')

cdslens = {}

# new method
# ----
# get location positions from CDS file if the sequence is complete at 3' (GeneID as key)
# get location positions from the RNA file if the sequence is complete at the 3'
# compare location lists, identify the 3' lengh differences based on that comparison
#   for the last start/stop location pair in the CDS location find the corrsponding start in the
#   RNA start/stop location pairs. Count the 3' differences in that pair and add that to the count
#   of the remaining location pairs in the RNA list. Confirm the position of the stop codons as well.
# return the corresponding final N characters of the mRNA sequence

# read CDS location data
cds_locations = {}
rna_locations = {}

for seq_record in SeqIO.parse(open(args.cds,'r'), args.format):
    gene_id = ""
    loclist = []
    # skip if the record is incomplete at the 3' end
    #if 'partial=3' in seq_record.description or 'partial=5\',3' in seq_record.description:
    if re.search(r'partial=[5\',3]', seq_record.description):
        continue
    for tempsplit in seq_record.description.split():
        if 'GeneID' in tempsplit:
            gene_id = tempsplit[tempsplit.find('GeneID:')+len('GeneID:'):-1]
        if 'location' in tempsplit:
            # remove brackets and ignore partial (>) as it would be 5' only
            tempsplit = re.sub(r'[()<>]','',tempsplit)
            # get list of locations (join for non-contiguous coordinates, else for contiguous)
            if 'join' in tempsplit:
                loclist = tempsplit[tempsplit.find('join')+len('join'):tempsplit.find(']')].split(',')
            else:
                loclist = re.sub(r'[\[\]a-zA-Z=]','',tempsplit).split(',')
            if 'location=complement' in tempsplit:
                loclist.reverse()
            for x in range(len(loclist)):
                loclist[x] = loclist[x].split('..')
            # some isomers have single coordinates (i.e., not start/stop location coordinates)
            # convert these to a pair [n,n]
            for x in loclist:
                if len(x) == 1:
                    x.append(x[0])
    if gene_id:
        cds_locations[gene_id] = loclist

for seq_record in SeqIO.parse(open(args.rna,'r'), args.format):
    gene_id = ""
    loclist = []
    utr_length = 0
    print(seq_record.id)
    if 'mRNA' not in seq_record.description:
        continue
    # skip if the record is incomplete at the 3' end
    if 'partial=3' in seq_record.description or 'partial=5\',3' in seq_record.description:
        continue
    for tempsplit in seq_record.description.split():
        if 'GeneID' in tempsplit:
            gene_id = tempsplit[tempsplit.find('GeneID:')+len('GeneID:'):-1]
        if 'location' in tempsplit:
            # remove brackets and ignore partial (>) as it would be 5' only
            tempsplit = re.sub(r'[()<>]','',tempsplit)
            # get list of locations
            if 'join' in tempsplit:
                loclist = tempsplit[tempsplit.find('join')+len('join'):tempsplit.find(']')].split(',')
            else:
                loclist = re.sub(r'[\[\]a-zA-Z=]','',tempsplit).split(',')
            if 'location=complement' in tempsplit:
                loclist.reverse()
            for x in range(len(loclist)):
                loclist[x] = loclist[x].split('..')

    # now process for CDS ending (i.e., finding the 3' UTR)
    # new approach
    # pseudocode:
    # if last element of rna is the same as the last element of the cds list then no UTR
    # else
    #   working backwards through the RNA location list
    #     if the start of the current RNA element is the same as the start of the last cds element
    #       add the difference between the end of the current rna element and end of the last cds element to the current utr length
    #       break out of loop returning UTR length
    #     else
    #     add the length of the current RNA elememt to the current UTR length
    #print(cds_locations[gene_id])
    #print(loclist)
    #print("{}\t{}".format(loclist[-1], cds_locations[gene_id]))
    if gene_id not in cds_locations:
        continue
    if cds_locations[gene_id][-1][1] == loclist[-1][1]:
        #print("they end in the same place")
        no_utr_handle.write('{}\n'.format(seq_record.id))
    else:
        for x in range(len(loclist)-1,-1,-1):
            #print("they do not end in the same place")
            #print("{}\t{}".format(loclist[x], cds_locations[gene_id][-1]))
            if loclist[x][0] == cds_locations[gene_id][-1][0]:
                #print("first elements match")
                #print("{}\t{}".format(loclist[x], cds_locations[gene_id][-1]))
                utr_length = utr_length + abs(int(loclist[x][1]) - int(cds_locations[gene_id][-1][1]))
                break
            utr_length = utr_length + abs(int(loclist[x][1]) - int(loclist[x][0])) + 1
    # end of new approach
    if utr_length > 0:
        output_handle.write('>{}\n{}\n'.format(seq_record.description,seq_record.seq[-utr_length:]))

output_handle.close()
no_utr_handle.close()

# end of new method
